@@ -49,15 +49,14 @@ NEGLIGENCE, OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION
 WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 */
 
-#include <kern/time_out.h>
+#include <kern/mach_clock.h>
 #include <i386/ipl.h>
+#include <i386/pic.h>
 #include <i386/pit.h>
 #include <i386/pio.h>
 
 int pitctl_port  = PITCTL_PORT;		/* For 386/20 Board */
 int pitctr0_port = PITCTR0_PORT;	/* For 386/20 Board */
-int pitctr1_port = PITCTR1_PORT;	/* For 386/20 Board */
-int pitctr2_port = PITCTR2_PORT;	/* For 386/20 Board */
 /* We want PIT 0 in square wave mode */
 
 int pit0_mode = PIT_C0|PIT_SQUAREMODE|PIT_READMODE ;
@@ -66,7 +65,7 @@ int pit0_mode = PIT_C0|PIT_SQUAREMODE|PIT_READMODE ;
 unsigned int clknumb = CLKNUM;		/* interrupt interval for timer 0 */
 
 void
-clkstart()
+clkstart(void)
 {
 	unsigned char	byte;
 	unsigned long s;
